@@ -11,23 +11,13 @@ from typing import TYPE_CHECKING, ClassVar
 from .args import parse_args
 from .aur import find_aur_packages, get_repo_url
 from .config import (
+    DECORATION,
     AurReposCachePath,
     BuildCachePath,
     BuildDepsLockPath,
     PackageCachePath,
     PikaurConfig,
-)
-from .core import (
-    PIPE,
-    DataType,
-    dirname,
-    interactive_spawn,
-    joined_spawn,
-    mkdir,
-    open_file,
-    remove_dir,
-    replace_file,
-    spawn,
+    UsingDynamicUsers,
 )
 from .exceptions import (
     BuildError,
@@ -39,10 +29,18 @@ from .exceptions import (
 )
 from .filelock import FileLock
 from .i18n import translate, translate_many
-from .logging import create_logger
+from .logging_extras import create_logger
 from .makepkg_config import MakePkgCommand, MakepkgConfig, get_pkgdest
+from .os_utils import (
+    chown_to_current,
+    dirname,
+    mkdir,
+    open_file,
+    remove_dir,
+    replace_file,
+)
 from .pacman import PackageDB, get_pacman_command, install_built_deps
-from .pprint import (
+from .pikaprint import (
     ColorsHighlight,
     TTYRestoreContext,
     bold_line,
@@ -52,10 +50,10 @@ from .pprint import (
     print_stderr,
     print_stdout,
 )
+from .pikatypes import AURInstallInfo, ComparableType
 from .privilege import (
     isolate_root_cmd,
     sudo,
-    using_dynamic_users,
 )
 from .prompt import (
     ask_to_continue,
@@ -63,6 +61,12 @@ from .prompt import (
     get_input,
     retry_interactive_command_or_exit,
 )
+from .spawn import (
+    PIPE,
+    interactive_spawn,
+    joined_spawn,
+    spawn,
+)
 from .srcinfo import SrcInfo
 from .updates import is_devel_pkg
 from .urllib_helper import wrap_proxy_env
@@ -72,8 +76,8 @@ if TYPE_CHECKING:
     from typing import Final
 
     from .args import PikaurArgs
-    from .core import InteractiveSpawn, SpawnArgs
     from .pacman import ProvidedDependency
+    from .spawn import InteractiveSpawn, SpawnArgs
 
 logger = create_logger("build")
 
@@ -95,7 +99,11 @@ def isolated_mkdir(to_path: Path) -> None:
     if mkdir_result.returncode != 0:
         print_stdout(mkdir_result.stdout_text)
         print_stderr(mkdir_result.stderr_text)
-        raise RuntimeError(translate(f"Can't create destination directory '{to_path}'."))
+        raise RuntimeError(
+            translate("Can't create destination directory '{to_path}'.").format(
+                to_path=to_path,
+            ),
+        )
 
 
 def copy_aur_repo(from_path: Path, to_path: Path) -> None:
@@ -120,16 +128,22 @@ def copy_aur_repo(from_path: Path, to_path: Path) -> None:
             isolated_mkdir(to_path)
         result = interactive_spawn(cmd_args)
         if result.returncode != 0:
-            raise RuntimeError(translate(f"Can't copy '{from_path}' to '{to_path}'."))
+            raise RuntimeError(
+                translate("Can't copy '{from_path}' to '{to_path}'.").format(
+                    from_path=from_path, to_path=to_path,
+                ),
+            )
 
 
-class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
+class PackageBuild(ComparableType):
     # pylint: disable=too-many-instance-attributes
-    clone = False
-    pull = False
+
+    clone: bool = False
+    pull: bool = False
 
     package_base: str
     package_names: list[str]
+    provides: list[str]
 
     repo_path: Path
     pkgbuild_path: Path
@@ -144,6 +158,7 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
     _build_files_copied = False
 
     failed: bool | None = None
+    is_dep: bool | None = None
 
     new_deps_to_install: list[str]
     new_make_deps_to_install: list[str]
@@ -162,12 +177,15 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
             f"{self.package_names}>"
         )
 
-    def __init__(  # pylint: disable=super-init-not-called
+    def __init__(
             self,
             package_names: list[str] | None = None,
             pkgbuild_path: str | None = None,
+            *,
+            is_dep: bool | None = None,
     ) -> None:
         self.args = parse_args()
+        self.is_dep = is_dep
 
         if pkgbuild_path:
             self.repo_path = dirname(pkgbuild_path)
@@ -179,13 +197,16 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
             if pkgbase and srcinfo.pkgnames:
                 self.package_names = package_names or srcinfo.pkgnames
                 self.package_base = pkgbase
+                self.provides = srcinfo.get_values("provides")
             else:
                 no_pkgname_error = translate("Can't get package name from PKGBUILD")
                 raise BuildError(message=no_pkgname_error, build=self)
         elif package_names:
             self.package_names = package_names
-            self.package_base = find_aur_packages([package_names[0]])[0][0].packagebase
-            self.repo_path = AurReposCachePath()() / self.package_base
+            aur_pkg = find_aur_packages([package_names[0]])[0][0]
+            self.package_base = aur_pkg.packagebase
+            self.provides = aur_pkg.provides
+            self.repo_path = AurReposCachePath() / self.package_base
             self.pkgbuild_path = self.repo_path / DEFAULT_PKGBUILD_BASENAME
         else:
             missing_property_error = translate(
@@ -196,7 +217,7 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
             )
             raise NotImplementedError(missing_property_error)
 
-        self.build_dir = BuildCachePath()() / self.package_base
+        self.build_dir = BuildCachePath() / self.package_base
         logger.debug("Build dir: {}", self.build_dir)
         self.build_gpgdir = self.args.build_gpgdir
         self.built_packages_paths = {}
@@ -295,6 +316,7 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
                 git_hash_path,
                 self.last_installed_file_path,
             )
+            chown_to_current(self.last_installed_file_path)
 
     @property
     def current_hash(self) -> str | None:
@@ -317,16 +339,17 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
             not (is_devel_pkg(self.package_base) and check_dev_pkgs)
         ):
             return
-        print_stdout("{} {}...".format(  # pylint: disable=consider-using-f-string
-            color_line("::", ColorsHighlight.white),
-            translate_many(
-                "Downloading the latest sources for a devel package {}",
-                "Downloading the latest sources for devel packages {}",
-                len(self.package_names),
-            ).format(
-                bold_line(", ".join(self.package_names)),
-            ),
-        ), tty_restore=tty_restore)
+        message = translate_many(
+            "Downloading the latest sources for a devel package {}",
+            "Downloading the latest sources for devel packages {}",
+            len(self.package_names),
+        ).format(
+            bold_line(", ".join(self.package_names)),
+        )
+        print_stdout(
+            f"{color_line(DECORATION, ColorsHighlight.white)} {message}...",
+            tty_restore=tty_restore,
+        )
         pkgver_result = joined_spawn(
             isolate_root_cmd(
                 [*MakePkgCommand.get(), "--nobuild", "--nocheck", "--nodeps"],
@@ -347,7 +370,7 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
                 answer = translate("a")
             else:  # pragma: no cover
                 prompt = "{} {}\n{}\n> ".format(
-                    color_line("::", ColorsHighlight.yellow),
+                    color_line(DECORATION, ColorsHighlight.yellow),
                     translate("Try recovering?"),
                     "\n".join((
                         translate("[R] retry clone"),
@@ -431,8 +454,10 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
             self,
             all_package_builds: dict[str, "PackageBuild"],
     ) -> None:
+        logger.debug("<< _FILTER_BUILT_DEPS")
 
         def _mark_dep_resolved(dep: str) -> None:
+            logger.debug("  _mark_dep_resolved: {}", dep)
             if dep in self.new_make_deps_to_install:
                 self.new_make_deps_to_install.remove(dep)
             if dep in self.new_deps_to_install:
@@ -444,31 +469,44 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
                 srcinfo = SrcInfo(
                     pkgbuild_path=pkg_build.pkgbuild_path, package_name=pkg_name,
                 )
-                all_provided_pkgnames.update({
-                    provided_name: pkg_name
-                    for provided_name in
-                    [pkg_name, *srcinfo.get_values("provides")]
-                })
+                stripped_pkg_name = VersionMatcher(pkg_name).pkg_name
+                all_provided_pkgnames.update(
+                    dict.fromkeys(
+                        [stripped_pkg_name, *(
+                            VersionMatcher(name).pkg_name
+                            for name in srcinfo.get_values("provides")
+                        )],
+                        stripped_pkg_name,
+                    ),
+                )
 
         self.built_deps_to_install = {}
 
+        logger.debug("  self.all_deps_to_install={}", self.all_deps_to_install)
+        logger.debug("  all_provided_pkgnames={}", all_provided_pkgnames)
         for dep in self.all_deps_to_install:
             dep_name = VersionMatcher(dep).pkg_name
+            logger.debug("    {} {}", dep, dep_name)
             if dep_name not in all_provided_pkgnames:
                 continue
             package_build = all_package_builds[all_provided_pkgnames[dep_name]]
+            logger.debug("    {} {}", package_build, all_provided_pkgnames[dep_name])
             if package_build == self:
                 _mark_dep_resolved(dep)
                 continue
             for pkg_name in package_build.package_names:
+                logger.debug("      {}", pkg_name)
                 if package_build.failed:
                     self.failed = True
+                    logger.debug("      FAILED")
                     raise DependencyError
                 if not package_build.built_packages_paths.get(pkg_name):
+                    logger.debug("      NOT_BUILT: {}", package_build.built_packages_paths)
                     raise DependencyNotBuiltYetError
                 self.built_deps_to_install[pkg_name] = \
                     package_build.built_packages_paths[pkg_name]
                 _mark_dep_resolved(dep)
+        logger.debug(">> _FILTER_BUILT_DEPS")
 
     def _get_pacman_command(self, ignore_args: list[str] | None = None) -> list[str]:
         return get_pacman_command(ignore_args=ignore_args) + (
@@ -484,11 +522,10 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
         if not self.built_deps_to_install:
             return
 
-        print_stderr("{} {}:".format(  # pylint: disable=consider-using-f-string
-            color_line("::", ColorsHighlight.purple),
-            translate("Installing already built dependencies for {}").format(
-                bold_line(", ".join(self.package_names))),
-        ))
+        message = translate("Installing already built dependencies for {}").format(
+            bold_line(", ".join(self.package_names)),
+        )
+        print_stderr(f"{color_line(DECORATION, ColorsHighlight.purple)} {message}:")
 
         try:
             update_self_deps = False
@@ -508,7 +545,7 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
         finally:
             PackageDB.discard_local_cache()
 
-    def _set_built_package_path(self) -> None:
+    def set_built_package_path(self) -> None:  # pylint: disable=too-many-branches
         pkg_paths_spawn = spawn(
             isolate_root_cmd(
                 [*MakePkgCommand.get(), "--packagelist"],
@@ -521,7 +558,14 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
         if not pkg_paths_spawn.stdout_text:
             return
         logger.debug("Package names: {}", pkg_paths_spawn)
-        pkg_paths = [Path(line) for line in pkg_paths_spawn.stdout_text.splitlines()]
+        pkg_paths: list[Path] = []
+        debug_pkg_paths: list[Path] = []
+        for line in pkg_paths_spawn.stdout_text.splitlines():
+            for pkg_name in self.package_names:
+                if f"{pkg_name}-debug-" in line:
+                    debug_pkg_paths.append(Path(line))
+                elif pkg_name in line:
+                    pkg_paths.append(Path(line))
         if not pkg_paths:
             return
         pkg_dest = get_pkgdest()
@@ -541,10 +585,12 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
             pkg_basename = Path(pkg_path).name
             logger.debug("Full path: {}, base path: {}", pkg_path, pkg_basename)
             if pkg_path == Path(pkg_basename):
-                pkg_path = (Path(pkg_dest) if pkg_dest else self.build_dir) / pkg_path
+                pkg_path = (
+                    Path(pkg_dest) if pkg_dest else self.build_dir
+                ) / pkg_path
                 logger.debug("Resolving full path: {} from base path: {}", pkg_path, pkg_basename)
             new_package_path = (
-                Path(pkg_dest) if pkg_dest else PackageCachePath()()
+                Path(pkg_dest) if pkg_dest else PackageCachePath()
             ) / pkg_basename
             logger.debug("New package path: {}", new_package_path)
             if not pkg_dest or MakePkgCommand.pkgdest_skipped:
@@ -552,30 +598,39 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
                 new_package_sig_path = new_package_path.parent / (
                     new_package_path.name + ".sig"
                 )
-                mkdir(PackageCachePath()())
-                replace_file(pkg_path, new_package_path)
-                replace_file(pkg_sig_path, new_package_sig_path)
+                mkdir(PackageCachePath())
+                for path_from, path_to in (
+                    (pkg_path, new_package_path),
+                    (pkg_sig_path, new_package_sig_path),
+                ):
+                    if path_from.exists():
+                        logger.debug("Copying {} to {}", path_from, path_to)
+                        replace_file(path_from, path_to)
+                logger.debug("Found debug packages: {}", debug_pkg_paths)
+                for debug_pkg_path in debug_pkg_paths:
+                    new_debug_pkg_path = new_package_path.parent / debug_pkg_path.name
+                    if debug_pkg_path.exists():
+                        logger.debug("Copying {} to {}", debug_pkg_path, new_debug_pkg_path)
+                        replace_file(debug_pkg_path, new_debug_pkg_path)
             pkg_path = new_package_path
             if pkg_path and pkg_path.exists():
                 self.built_packages_paths[pkg_name] = pkg_path
 
     def check_if_already_built(self) -> bool:
         self.get_latest_dev_sources()
-        self._set_built_package_path()
+        self.set_built_package_path()
         if (
-                not self.args.rebuild and
+                not (self.args.rebuild and not self.is_dep) and
                 len(self.built_packages_paths) == len(self.package_names)
         ):
-            print_stderr("{} {}\n".format(  # pylint: disable=consider-using-f-string
-                color_line("::", ColorsHighlight.green),
-                translate_many(
-                    "Package {pkg} is already built. Pass '--rebuild' flag to force the build.",
-                    "Packages {pkg} are already built. Pass '--rebuild' flag to force the build.",
-                    len(self.package_names),
-                ).format(
-                    pkg=bold_line(", ".join(self.package_names)),
-                ),
-            ))
+            message = translate_many(
+                "Package {pkg} is already built. Pass '--rebuild' flag to force the build.",
+                "Packages {pkg} are already built. Pass '--rebuild' flag to force the build.",
+                len(self.package_names),
+            ).format(
+                pkg=bold_line(", ".join(self.package_names)),
+            )
+            print_stderr(f"{color_line(DECORATION, ColorsHighlight.green)} {message}\n")
             return True
         return False
 
@@ -644,11 +699,10 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
         if not self.all_deps_to_install:
             return
 
-        print_stderr("{} {}:".format(  # pylint: disable=consider-using-f-string
-            color_line("::", ColorsHighlight.purple),
-            translate("Installing repository dependencies for {}").format(
-                bold_line(", ".join(self.package_names))),
-        ))
+        message = translate("Installing repository dependencies for {}").format(
+            bold_line(", ".join(self.package_names)),
+        )
+        print_stderr(f"{color_line(DECORATION, ColorsHighlight.purple)} {message}:")
 
         # @TODO: add support for --skip-failed-build here:
         retry_interactive_command_or_exit(
@@ -660,20 +714,23 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
             pikspect=True,
             conflicts=self.resolved_conflicts,
         )
-        PackageDB.discard_local_cache()
-        self._local_pkgs_with_build_deps = set(PackageDB.get_local_dict().keys())
-        self._local_provided_pkgs_with_build_deps = PackageDB.get_local_provided_dict()
 
     def install_all_deps(self, all_package_builds: dict[str, "PackageBuild"]) -> None:
-        with FileLock(BuildDepsLockPath()()):
+        with FileLock(BuildDepsLockPath()):
             self.get_deps(all_package_builds)
             if self.all_deps_to_install or self.built_deps_to_install:
                 PackageDB.discard_local_cache()
                 self._local_pkgs_wo_build_deps = set(PackageDB.get_local_dict().keys())
             self.install_built_deps(all_package_builds)
             self._install_repo_deps()
+            PackageDB.discard_local_cache()
+            self._local_pkgs_with_build_deps = set(PackageDB.get_local_dict().keys())
+            self._local_provided_pkgs_with_build_deps = PackageDB.get_local_provided_dict()
 
     def _remove_installed_deps(self) -> None:
+        # logger.debug(
+        #     "Local pkgs before installing build deps: {}", self._local_pkgs_wo_build_deps,
+        # )
         if not self._local_pkgs_wo_build_deps:
             return
 
@@ -718,11 +775,10 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
         if not deps_packages_installed or self.args.keepbuilddeps:
             return
 
-        print_stderr("{} {}:".format(  # pylint: disable=consider-using-f-string
-            color_line("::", ColorsHighlight.purple),
-            translate("Removing already installed dependencies for {}").format(
-                bold_line(", ".join(self.package_names))),
-        ))
+        message = translate("Removing already installed dependencies for {}").format(
+            bold_line(", ".join(self.package_names)),
+        )
+        print_stderr(f"{color_line(DECORATION, ColorsHighlight.purple)} {message}:")
         retry_interactive_command_or_exit(
             sudo(
                 # pacman --remove flag conflicts with some --sync options:
@@ -808,21 +864,24 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
         print_stdout()
         return result
 
-    def build_with_makepkg(self) -> bool:  # pylint: disable=too-many-branches,too-many-statements
+    def build_with_makepkg(  # pylint: disable=too-many-branches,too-many-statements
+            self,
+            *,
+            skip_check: bool = False,
+    ) -> bool:
         makepkg_args = []
         if not self.args.needed:
             makepkg_args.append("--force")
         if not color_enabled():
             makepkg_args.append("--nocolor")
 
-        print_stderr("\n{} {}:".format(  # pylint: disable=consider-using-f-string
-            color_line("::", ColorsHighlight.purple),
-            translate("Starting the build"),
-        ))
+        message = translate("Starting the build")
+        print_stderr(
+            f"\n{color_line(DECORATION, ColorsHighlight.purple)} {message}:",
+        )
         build_succeeded = False
         skip_pgp_check = False
         skip_file_checksums = False
-        skip_check = False
         no_prepare = False
         while True:
             result = self._run_makepkg_cmd(
@@ -852,7 +911,7 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
                 answer = translate("a")
             else:  # pragma: no cover
                 prompt = "{} {}\n{}\n> ".format(
-                    color_line("::", ColorsHighlight.yellow),
+                    color_line(DECORATION, ColorsHighlight.yellow),
                     translate("Try recovering?"),
                     "\n".join((
                         translate("[R] retry build"),
@@ -927,6 +986,7 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
             self,
             all_package_builds: dict[str, "PackageBuild"],
             resolved_conflicts: list[list[str]],
+            skip_checkfunc_for_pkgnames: list[str],
     ) -> None:
         self.resolved_conflicts = resolved_conflicts
 
@@ -934,14 +994,22 @@ class PackageBuild(DataType):  # pylint: disable=too-many-public-methods
 
         self.install_all_deps(all_package_builds)
         try:
-            build_succeeded = True if self.check_if_already_built() else self.build_with_makepkg()
+            skip_check = False
+            for pkg_name in self.package_names:
+                if pkg_name in skip_checkfunc_for_pkgnames:
+                    skip_check = True
+            build_succeeded = (
+                True
+                if self.check_if_already_built()
+                else self.build_with_makepkg(skip_check=skip_check)
+            )
         finally:
             self._remove_installed_deps()
 
         if not build_succeeded:
             self.failed = True
             raise BuildError(message="failed to build", build=self)
-        self._set_built_package_path()
+        self.set_built_package_path()
 
 
 class AlreadyClonedRepos:
@@ -957,21 +1025,26 @@ class AlreadyClonedRepos:
         return repo in cls.repos
 
 
-def clone_aur_repos(package_names: list[str]) -> dict[str, PackageBuild]:
+def clone_aur_repos(package_infos: list[AURInstallInfo]) -> dict[str, PackageBuild]:
+    package_names = [info.name for info in package_infos]
     aur_pkgs, _ = find_aur_packages(package_names)
     packages_bases: dict[str, list[str]] = {}
     for aur_pkg in aur_pkgs:
         packages_bases.setdefault(aur_pkg.packagebase, []).append(aur_pkg.name)
     package_builds_by_base = {
-        pkgbase: PackageBuild(pkg_names)
+        pkgbase: PackageBuild(package_names=pkg_names)
         for pkgbase, pkg_names in packages_bases.items()
         if not AlreadyClonedRepos.get(pkgbase)
     }
+    install_infos_by_base = {
+        info.package.packagebase: info
+        for info in package_infos
+    }
 
     pool_size: int | None = None
     if clone_c := parse_args().aur_clone_concurrency:
         pool_size = clone_c
-    elif using_dynamic_users():
+    elif UsingDynamicUsers():
         pool_size = 1
     exc: CloneError | None = None
     with (
@@ -997,7 +1070,10 @@ def clone_aur_repos(package_names: list[str]) -> dict[str, PackageBuild]:
         raise exc
 
     all_package_builds_by_base = {
-        pkgbase: PackageBuild(pkg_names)
+        pkgbase: PackageBuild(
+            package_names=pkg_names,
+            is_dep=bool(install_infos_by_base[pkgbase].required_by),
+        )
         for pkgbase, pkg_names in packages_bases.items()
     }
     return {
