@@ -3,14 +3,16 @@ from pathlib import Path
 from typing import TYPE_CHECKING
 
 from .args import parse_args
-from .aur import AURPackageInfo, find_aur_packages, get_repo_url
+from .aur import find_aur_packages, get_repo_url
 from .aur_deps import get_aur_deps_list
-from .core import check_runtime_deps, interactive_spawn
 from .exceptions import PackagesNotFoundInRepoError
 from .i18n import translate
+from .os_utils import check_executables
 from .pacman import PackageDB
-from .pprint import print_stdout
+from .pikaprint import print_stdout
+from .pikatypes import AURPackageInfo
 from .print_department import print_not_found_packages
+from .spawn import interactive_spawn
 from .urllib_helper import wrap_proxy_env
 
 if TYPE_CHECKING:
@@ -22,19 +24,7 @@ def clone_aur_pkgs(aur_pkgs: list[AURPackageInfo], pwd: Path) -> None:
         name = aur_pkg.name
         repo_path = pwd / name
         print_stdout()
-        action = "clone"
         if repo_path.exists():
-            action = "update"
-        if action == "clone":
-            interactive_spawn(
-                wrap_proxy_env([
-                    "git",
-                    "clone",
-                    get_repo_url(aur_pkg.packagebase),
-                    str(repo_path),
-                ]),
-            )
-        elif action == "update":
             interactive_spawn([
                 "git",
                 "-C", repo_path.as_posix(),
@@ -43,19 +33,35 @@ def clone_aur_pkgs(aur_pkgs: list[AURPackageInfo], pwd: Path) -> None:
                 "master",
             ])
         else:
-            raise NotImplementedError
+            interactive_spawn(
+                wrap_proxy_env([
+                    "git",
+                    "clone",
+                    get_repo_url(aur_pkg.packagebase),
+                    str(repo_path),
+                ]),
+            )
 
 
 def clone_repo_pkgs(repo_pkgs: list["pyalpm.Package"], pwd: Path) -> None:
     for repo_pkg in repo_pkgs:
-        name = repo_pkg.name
+        name = repo_pkg.base
         repo_path = pwd / name
-        action = "clone"
-        if repo_path.exists():
-            action = "update"
         print_stdout()
-        print_stdout(translate(f"Package '{name}' going to be cloned into '{repo_path}'..."))
-        if action == "clone":
+        print_stdout(
+            translate("Package '{name}' going to be cloned into '{repo_path}'...").format(
+                name=name, repo_path=repo_path,
+            ),
+        )
+        if repo_path.exists():
+            interactive_spawn([
+                "git",
+                "-C", repo_path.as_posix(),
+                "pull",
+                "origin",
+                "main",
+            ])
+        else:
             interactive_spawn(
                 [
                     "pkgctl",
@@ -66,16 +72,6 @@ def clone_repo_pkgs(repo_pkgs: list["pyalpm.Package"], pwd: Path) -> None:
                 ],
                 cwd=pwd,
             )
-        elif action == "update":
-            interactive_spawn([
-                "git",
-                "-C", repo_path.as_posix(),
-                "pull",
-                "origin",
-                "main",
-            ])
-        else:
-            raise NotImplementedError
 
 
 def cli_getpkgbuild() -> None:
@@ -95,13 +91,13 @@ def cli_getpkgbuild() -> None:
             repo_pkgs.append(repo_pkg)
 
     if repo_pkgs:
-        check_runtime_deps(["pkgctl"])
+        check_executables(["pkgctl"])
 
     if not_found_repo_pkgs:
         print_not_found_packages(not_found_repo_pkgs)
 
     if args.deps:
-        aur_pkgs = aur_pkgs + get_aur_deps_list(aur_pkgs)
+        aur_pkgs += get_aur_deps_list(aur_pkgs)
 
     clone_aur_pkgs(aur_pkgs=aur_pkgs, pwd=pwd)
     clone_repo_pkgs(repo_pkgs=repo_pkgs, pwd=pwd)
