@@ -2,23 +2,25 @@
 
 import fnmatch
 import re
-from abc import ABCMeta, abstractmethod
+from abc import ABC, abstractmethod
+from dataclasses import dataclass
 from typing import TYPE_CHECKING, ClassVar
 
 import pyalpm
-from pycman.config import PacmanConfig as PycmanConfig
 
-from .args import PACMAN_APPEND_OPTS, PACMAN_STR_OPTS, parse_args, reconstruct_args
-from .config import PikaurConfig
-from .core import DataType, PackageSource, spawn
+from .alpm import PyAlpmWrapper
+from .args import PACMAN_APPEND_OPTS, get_pacman_str_opts, parse_args, reconstruct_args
 from .exceptions import DependencyError, PackagesNotFoundInRepoError
 from .i18n import translate
 from .lock import FancyLock
-from .logging import create_logger
+from .logging_extras import create_logger
 from .pacman_i18n import _p
-from .pprint import color_enabled, print_error, print_stderr
+from .pikaprint import color_enabled, print_error, print_stderr
+from .pikatypes import AnyPackage, PackageSource
 from .privilege import sudo
 from .prompt import retry_interactive_command, retry_interactive_command_or_exit
+from .provider import Provider
+from .spawn import spawn
 from .version import VersionMatcher
 
 if TYPE_CHECKING:
@@ -27,20 +29,6 @@ if TYPE_CHECKING:
     from re import Pattern
     from typing import Final
 
-    from .aur import AURPackageInfo
-
-
-OFFICIAL_REPOS: "Final" = (
-    "core",
-    "extra",
-    "multilib",
-    "core-testing",
-    "extra-testing",
-    "multilib-testing",
-    "core-staging",
-    "extra-staging",
-)
-
 
 REPO_NAME_DELIMITER: "Final" = "/"
 
@@ -64,57 +52,71 @@ def create_pacman_pattern(pacman_message: str) -> "Pattern[str]":
     )
 
 
-def get_pacman_command(ignore_args: list[str] | None = None) -> list[str]:
+def get_pacman_command(  # pylint: disable=too-many-branches
+        ignore_args: list[str] | None = None,
+) -> list[str]:
     ignore_args = ignore_args or []
     args = parse_args()
-    pacman_path = PikaurConfig().misc.PacmanPath.get_str()
-    pacman_cmd = [pacman_path]
+    pacman_cmd = [args.pacman_path]
     if color_enabled():
         pacman_cmd += ["--color=always"]
     else:
         pacman_cmd += ["--color=never"]
 
-    for _short, arg, _default, _help in PACMAN_STR_OPTS:
-        if arg in ["color"]:  # we force it anyway
+    for data in get_pacman_str_opts():
+        if data.help_only:
+            continue
+        arg = data.long or data.short
+        if not arg:
+            continue
+        if arg == "color":  # we force it anyway
             continue
         if arg in ignore_args:
             continue
         value = getattr(args, arg)
         if value:
-            pacman_cmd += ["--" + arg, value]
+            if data.long:
+                pacman_cmd += ["--" + data.long, value]
+            elif data.short:
+                pacman_cmd += ["-" + data.short, value]
 
-    for _short, arg, _default, _help in PACMAN_APPEND_OPTS:
-        if arg in ["ignore"]:  # we reprocess it anyway
+    for data in PACMAN_APPEND_OPTS:
+        if data.help_only:
+            continue
+        arg = data.long or data.short
+        if not arg:
+            continue
+        if arg == "ignore":  # we reprocess it anyway
             continue
         if arg in ignore_args:
             continue
         for value in getattr(args, arg) or []:
-            pacman_cmd += ["--" + arg, value]
+            if data.long:
+                pacman_cmd += ["--" + data.long, value]
+            elif data.short:
+                pacman_cmd += ["-" + data.short, value]
 
     return pacman_cmd
 
 
-class RawPrintFormat(DataType):
+@dataclass
+class RawPrintFormat:
 
     returncode: int
-    stdout_text: str
-    stderr_text: str
+    stdout_text: str | None
+    stderr_text: str | None
 
 
-class PacmanPrint(DataType):
+@dataclass
+class PacmanPrint:
 
     full_name: str
     repo: str
     name: str
 
 
-class PacmanConfig(PycmanConfig):
-
-    def __init__(self) -> None:
-        super().__init__(conf=parse_args().config or "/etc/pacman.conf")
-
-
-class ProvidedDependency(DataType):
+@dataclass
+class ProvidedDependency:
     name: str
     package: pyalpm.Package
     version_matcher: VersionMatcher
@@ -126,7 +128,7 @@ class ProvidedDependency(DataType):
         )
 
 
-def get_pkg_id(pkg: "AURPackageInfo | pyalpm.Package") -> str:
+def get_pkg_id(pkg: AnyPackage) -> str:
     if isinstance(pkg, pyalpm.Package):
         return f"{pkg.db.name}/{pkg.name}"
     return f"aur/{pkg.name}"
@@ -144,7 +146,7 @@ def get_db_lock(package_source: PackageSource) -> type[DbLockRepo | DbLockLocal]
     return DbLockRepo if package_source is PackageSource.REPO else DbLockLocal
 
 
-class PackageDBCommon(metaclass=ABCMeta):
+class PackageDBCommon(ABC):
 
     _packages_list_cache: ClassVar[dict[PackageSource, list[pyalpm.Package]]] = {}
     _packages_dict_cache: ClassVar[dict[PackageSource, dict[str, pyalpm.Package]]] = {}
@@ -269,23 +271,12 @@ class RepositoryNotFoundError(Exception):
     pass
 
 
-class PackageDB(PackageDBCommon):
-
-    _alpm_handle: pyalpm.Handle | None = None
+class PackageDB(PackageDBCommon, PyAlpmWrapper):
 
     _pacman_pformat_cache: ClassVar[dict[str, RawPrintFormat]] = {}
     _pacman_test_cache: ClassVar[dict[str, list[VersionMatcher]]] = {}
     _pacman_repo_pkg_present_cache: ClassVar[dict[str, bool]] = {}
 
-    @classmethod
-    def get_alpm_handle(cls) -> pyalpm.Handle:
-        if not cls._alpm_handle:
-            cls._alpm_handle = PacmanConfig().initialize_alpm()
-        if not cls._alpm_handle:
-            cant_init_alpm = translate("Cannot initialize ALPM")
-            raise RuntimeError(cant_init_alpm)
-        return cls._alpm_handle
-
     @classmethod
     def discard_local_cache(cls) -> None:
         super().discard_local_cache()
@@ -329,7 +320,7 @@ class PackageDB(PackageDBCommon):
         if not cls._provided_dict_cache.get(package_source):
             provided_pkg_names = super().get_provided_dict(package_source)
             if package_source == PackageSource.REPO:
-                for _what_provides, provided_pkgs in provided_pkg_names.items():
+                for provided_pkgs in provided_pkg_names.values():
                     provided_pkgs.sort(
                         key=lambda p: f"{cls.get_repo_priority(p.package.db.name)}{p.package.name}",
                     )
@@ -340,26 +331,27 @@ class PackageDB(PackageDBCommon):
     def search_repo(
             cls,
             search_query: str,
-            db_name: str | None = None,
+            db_names: list[str] | None = None,
             *,
             names_only: bool = False,
             exact_match: bool = False,
     ) -> list[pyalpm.Package]:
         if REPO_NAME_DELIMITER in search_query:
             db_name, search_query = search_query.split(REPO_NAME_DELIMITER)
-        result = []
-        for sync_db in reversed(cls.get_alpm_handle().get_syncdbs()):
-            if not db_name or db_name == sync_db.name:
-                for pkg in sync_db.search(search_query):
-                    if (
-                            (
-                                not names_only or search_query in pkg.name
-                            ) and (
-                                not exact_match or search_query in ([pkg.name, *pkg.groups])
-                            )
-                    ):
-                        result.append(pkg)
-        return list(reversed(result))
+            db_names = [db_name]
+        return list(reversed([
+            pkg
+            for sync_db in reversed(cls.get_alpm_handle().get_syncdbs())
+            if not db_names or (sync_db.name in db_names)
+            for pkg in sync_db.search(search_query)
+            if (
+                (
+                    not names_only or search_query in pkg.name
+                ) and (
+                    not exact_match or search_query in ({pkg.name, *pkg.groups})
+                )
+            )
+        ]))
 
     @classmethod
     def get_repo_list(cls, *, quiet: bool = False) -> list[pyalpm.Package]:
@@ -374,13 +366,15 @@ class PackageDB(PackageDBCommon):
 
     @classmethod
     def get_last_installed_package_date(cls) -> int:
-        repo_names = []
-        for repo in PackageDB.get_repo_list():
-            repo_names.append(repo.name)
-        packages = []
-        for package in PackageDB.get_local_list():
-            if package.name in repo_names:
-                packages.append(package)
+        repo_names = [
+            repo.name
+            for repo in PackageDB.get_repo_list()
+        ]
+        packages = [
+            package
+            for package in PackageDB.get_local_list()
+            if package.name in repo_names
+        ]
         packages_by_date = sorted(packages, key=lambda x: -x.installdate)
         return int(packages_by_date[0].installdate)
 
@@ -435,10 +429,18 @@ class PackageDB(PackageDBCommon):
 
     @classmethod
     def get_sync_print_format_output(
-            cls, pkg_names: list[str], *, check_deps: bool = True, package_only: bool = False,
+            cls, pkg_names: list[str],
+            *,
+            check_deps: bool = True, package_only: bool = False, skip_dep_test: bool = False,
     ) -> list[PacmanPrint]:
         return cls.get_print_format_output(
-            [*get_pacman_command(), "--sync", *pkg_names],
+            [
+                *get_pacman_command(),
+                "--sync",
+                *pkg_names,
+            ] + (
+                ["--nodeps"] if skip_dep_test else []
+            ),
             check_deps=check_deps,
             package_only=package_only,
         )
@@ -485,8 +487,12 @@ class PackageDB(PackageDBCommon):
             if pkg_name not in cls._pacman_repo_pkg_present_cache:
                 pkg_names_to_check += pkg_name.split(",")
             elif not cls._pacman_repo_pkg_present_cache[pkg_name]:
-                not_found_pkg_names.append(pkg_name)
+                not_found_pkg_names.append(VersionMatcher(pkg_name).pkg_name)
 
+        logger.debug(
+            "get_not_found_repo_packages: pkg_names_to_check={} not_found_pkg_names={}",
+            pkg_names_to_check, not_found_pkg_names,
+        )
         if not pkg_names_to_check:
             return not_found_pkg_names
 
@@ -503,10 +509,13 @@ class PackageDB(PackageDBCommon):
                     pkg_name = VersionMatcher(groups[0]).pkg_name
                     new_not_found_pkg_names.append(pkg_name)
 
+        end_result = not_found_pkg_names + new_not_found_pkg_names
         for pkg_name in pkg_names_to_check:
-            cls._pacman_repo_pkg_present_cache[pkg_name] = pkg_name not in new_not_found_pkg_names
+            cls._pacman_repo_pkg_present_cache[pkg_name] = (
+                VersionMatcher(pkg_name).pkg_name not in end_result
+            )
 
-        return not_found_pkg_names + new_not_found_pkg_names
+        return end_result
 
     @classmethod
     def get_not_found_local_packages(cls, pkg_lines: list[str]) -> list[str]:
@@ -521,7 +530,7 @@ class PackageDB(PackageDBCommon):
 
     @classmethod
     def find_repo_package(cls, pkg_name: str) -> pyalpm.Package:
-        # @TODO: interactively ask for multiple providers and save the answer?
+
         if cls._pacman_repo_pkg_present_cache.get(pkg_name) is False:
             raise PackagesNotFoundInRepoError(packages=[pkg_name])
         all_repo_pkgs = PackageDB.get_repo_dict()
@@ -537,16 +546,26 @@ class PackageDB(PackageDBCommon):
             return found_pkgs[0]
 
         pkg_name = VersionMatcher(pkg_name).pkg_name
-        for pkg in found_pkgs:
-            if pkg.name == pkg_name:
-                return pkg
+        matching_pkgs: list[pyalpm.Package] = [
+            pkg
+            for pkg in found_pkgs
+            if pkg.name == pkg_name
+        ]
         for pkg in found_pkgs:
             if pkg.provides:
                 for provided_pkg_line in pkg.provides:
                     provided_name = VersionMatcher(provided_pkg_line).pkg_name
                     if provided_name == pkg_name:
-                        return pkg
-        raise PackagesNotFoundInRepoError(packages=[pkg_name])
+                        matching_pkgs.append(pkg)
+        if not matching_pkgs:
+            raise PackagesNotFoundInRepoError(packages=[pkg_name])
+        return Provider.choose(
+            dependency=pkg_name, options=matching_pkgs,
+        )
+
+    @classmethod
+    def get_local_pkg_uncached(cls, name: str) -> pyalpm.Package | None:
+        return cls.get_alpm_handle().get_localdb().get_pkg(name)
 
 
 def get_upgradeable_package_names() -> list[str]:
@@ -558,10 +577,12 @@ def get_upgradeable_package_names() -> list[str]:
     return upgradeable_packages_output.splitlines()
 
 
-def find_upgradeable_packages() -> list[pyalpm.Package]:
+def find_upgradeable_packages(*, skip_dep_test: bool = False) -> list[pyalpm.Package]:
+    logger.debug("< FIND_UPGRADEABLE_PACKAGES")
     all_repo_pkgs = PackageDB.get_repo_dict()
 
     pkg_names = get_upgradeable_package_names()
+    logger.debug("upgradeable_package_names={}", pkg_names)
     if not pkg_names:
         return []
 
@@ -577,8 +598,12 @@ def find_upgradeable_packages() -> list[pyalpm.Package]:
             try:
                 results += PackageDB.get_sync_print_format_output([pkg_name])
             except DependencyError as exc2:
-                print_error(translate("Because of:"))
+                print_stderr(translate("Because of:") + f" {pkg_name}")
                 print_stderr(str(exc2))
+    if skip_dep_test and (len(results) < len(pkg_names)):
+        results = PackageDB.get_sync_print_format_output(
+            pkg_names=pkg_names, skip_dep_test=skip_dep_test,
+        )
     return [
         all_repo_pkgs[result.full_name] for result in results
         if result.name in all_local_pkgs
@@ -593,11 +618,12 @@ def find_sysupgrade_packages(
 
     extra_args: list[str] = []
     for excluded_pkg_name in ignore_pkgs or []:
-        extra_args.append("--ignore")
         # pacman's --ignore doesn't work with repo name:
-        extra_args.append(strip_repo_name(excluded_pkg_name))
-    for added_pkg_name in install_pkgs or []:
-        extra_args.append(added_pkg_name)
+        extra_args.extend(("--ignore", strip_repo_name(excluded_pkg_name)))
+    extra_args.extend(
+        added_pkg_name
+        for added_pkg_name in install_pkgs or []
+    )
 
     logger.debug("Gonna get sysupgrade info...")
     results = PackageDB.get_print_format_output(
@@ -611,13 +637,12 @@ def find_sysupgrade_packages(
 
 
 def find_packages_not_from_repo() -> list[str]:
-    local_pkg_names = PackageDB.get_local_pkgnames()
     repo_pkg_names = PackageDB.get_repo_pkgnames()
-    not_found_packages = []
-    for pkg_name in local_pkg_names:
-        if pkg_name not in repo_pkg_names:
-            not_found_packages.append(pkg_name)
-    return not_found_packages
+    return [
+        pkg_name
+        for pkg_name in PackageDB.get_local_pkgnames()
+        if pkg_name not in repo_pkg_names
+    ]
 
 
 def refresh_pkg_db_if_needed() -> None:
@@ -651,7 +676,7 @@ def install_built_deps(
         ])
 
     explicitly_installed_deps = []
-    for pkg_name, _path in deps_names_and_paths.items():
+    for pkg_name in deps_names_and_paths:
         logger.debug(pkg_name)
         if pkg_name in local_packages and local_packages[pkg_name].reason == 0:
             explicitly_installed_deps.append(pkg_name)
