@@ -3,13 +3,14 @@
 import shutil
 from pathlib import Path
 
-from .config import BuildCachePath, CacheRoot
-from .core import chown_to_current, open_file, spawn
+from .config import BuildCachePath, CacheRoot, UsingDynamicUsers
 from .exceptions import SysExit
 from .i18n import translate
 from .makepkg_config import MakePkgCommand, MakepkgConfig
-from .pprint import print_error, print_stderr
-from .privilege import isolate_root_cmd, using_dynamic_users
+from .os_utils import chown_to_current, open_file
+from .pikaprint import print_error, print_stderr
+from .privilege import isolate_root_cmd
+from .spawn import spawn
 from .version import VersionMatcher
 
 
@@ -65,13 +66,13 @@ class SrcInfo:
 
     def get_values(self, field: str, lines: list[str] | None = None) -> list[str]:
         prefix = field + " = "
-        values = []
         if lines is None:
             lines = self._common_lines + self._package_lines
-        for line in lines:
-            if line.strip().startswith(prefix):
-                values.append(line.strip().split(prefix)[1])
-        return values
+        return [
+            line.strip().split(prefix)[1]
+            for line in lines
+            if line.strip().startswith(prefix)
+        ]
 
     def get_value(self, field: str, fallback: str | None = None) -> str | None:
         values = self.get_values(field)
@@ -106,7 +107,7 @@ class SrcInfo:
     def _get_build_depends(self, field: str) -> dict[str, VersionMatcher]:
         return self._get_depends(field=field, lines=self._common_lines)
 
-    def get_depends(self) -> dict[str, VersionMatcher]:
+    def get_runtime_depends(self) -> dict[str, VersionMatcher]:
         return self._get_depends("depends")
 
     def get_build_depends(self) -> dict[str, VersionMatcher]:
@@ -127,8 +128,8 @@ class SrcInfo:
 
     def regenerate(self) -> None:
         working_directory = self.repo_path
-        if using_dynamic_users() and not str(self.repo_path).startswith(str(CacheRoot()())):
-            working_directory = BuildCachePath()() / (
+        if UsingDynamicUsers() and not str(self.repo_path).startswith(str(CacheRoot())):
+            working_directory = BuildCachePath() / (
                 "_info_" + (self.get_value("pkgbase") or "unknown")
             )
             if not working_directory.exists():
